package org.garret.metronome;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import com.nokia.mid.sound.Sound;

public class Metronome extends MIDlet
{ 
    static final Command QUIT_CMD = new Command(Locale.QUIT, Command.BACK, 3);
    static final Command BACK_CMD = new Command(Locale.BACK, Command.BACK, 2);
    static final Command SAVE_CMD = new Command(Locale.SAVE, Command.OK, 1);
    static final Command SETTINGS_CMD = new Command(Locale.SETTINGS, Command.ITEM, 1);
    static final Command HELP_CMD = new Command(Locale.HELP, Command.ITEM, 2);

    int             rate;
    int             frequency;
    int             duration;
    int             volume;
    int             flags;
    RecordStore     store;
    Sound           sound;

    public MetronomeCanvas  canvas;
    public static Metronome instance;

    static final String STORE_NAME = "METRONOME";
    static final int RECORD_ID = 1;

    static final int VIBRA = 1;
    static final int BACKLIGHT = 2;

    static final int DEFAULT_FREQUENCY = 440;
    static final int DEFAULT_DURATION = 100;
    static final int DEFAULT_VOLUME = 255;
    static final int DEFAULT_FLAGS = VIBRA|BACKLIGHT;


    public Metronome() {
        instance = this;
        try { 
            try { 
                store = RecordStore.openRecordStore(STORE_NAME, false);
                loadSettings();
            } catch (RecordStoreNotFoundException x) {             
                store = RecordStore.openRecordStore(STORE_NAME, true);
                initializeStore();
            }             
        } catch(Exception x) { 
        }
        initSound();
    }

    void initSound() { 
        sound = new Sound(frequency, duration);
        sound.setGain(volume);
        if (canvas != null) {
            sound.setSoundListener(canvas);
        }
    }

    void loadSettings() { 
        try { 
            byte[] record = store.getRecord(RECORD_ID);
            frequency = ((record[0] & 0xFF) << 8) | (record[1] & 0xFF);
            duration = ((record[2] & 0xFF) << 8) | (record[3] & 0xFF);
            rate = ((record[4] & 0xFF) << 8) | (record[5] & 0xFF);            
            volume = record[6] & 0xFF;
            flags = record[7];
        } catch (Exception x) {             
            initializeStore();
        }
    }

    void initializeStore() { 
        frequency = DEFAULT_FREQUENCY;
        duration = DEFAULT_DURATION;
        volume = DEFAULT_VOLUME;
        flags = DEFAULT_FLAGS;
        rate = 0;
        byte[] record = packSettings();
        try { 
            store.addRecord(record, 0, record.length);
        } catch (Exception x) {}
    }
        
    void saveSettings() {
        byte[] record = packSettings();
        try { 
            store.setRecord(RECORD_ID, record, 0, record.length);
        } catch (Exception x) {}
    }
    
    byte[] packSettings() { 
        byte[] record = new byte[8];
        record[0] = (byte)(frequency >> 8);
        record[1] = (byte)frequency;
        record[2] = (byte)(duration >> 8);
        record[3] = (byte)duration;
        record[4] = (byte)(rate >> 8);
        record[5] = (byte)rate;
        record[6] = (byte)volume;
        record[7] = (byte)flags;
        return record;
    }

    protected void destroyApp(boolean unconditional) 
    {
        try { 
            store.closeRecordStore();
        } catch (Exception x) {}
    }  

    protected  void pauseApp() 
    {
    }

    protected void startApp() 
    {
        canvas = new MetronomeCanvas(this);
    }
}
